# soubor utils.tcl

proc kdMovePart {dx dy w tagid idx} {
    set coords [$w coords $tagid]
    if {$idx == -1} then {
	set idx [expr [llength $coords] -2]
    }
    set x0 [expr [lindex $coords $idx] + $dx]
    set y0 [expr [lindex $coords [expr $idx +1]] + $dy]
    eval $w coords $tagid [lreplace $coords $idx [expr $idx +1] $x0 $y0]
}

proc kdNearest {x y coords} {
    set min 20
    set mi no
    set li 0 
    while { $coords != "" } {
	set d [expr  abs([lindex $coords 0] - $x) + \
		abs([lindex $coords 1] - $y)]
	if { $d < $min } { 
	    set min $d
	    set mi $li
	}
	set coords [lreplace $coords 0 1]
	incr li 2
    }
    return $mi
}


proc kdSaveBind {w events} {
    global kdPriv
    foreach e $events {
	set kdPriv(bind,$w,$e) [bind $w $e]
    }
}

proc kdRestoreBind {w events} {
    global kdPriv
    foreach e $events {
	bind $w $e $kdPriv(bind,$w,$e)
    }
}

proc kdSaveProcs procs {
    set saved ""
    foreach p $procs {
	if ![catch {rename $p kd_saved_proc_$p}] {
	    lappend saved $p
    }   }
    return $saved
}

proc kdRestoreProcs procs {
    foreach p $procs {
	rename $p ""
	catch {rename kd_saved_proc_$p $p}
    }
}

proc kdIntersecOval {oval lx ly &xv &yv} {
    # oval = souradnice ovalu
    # lx,ly = predposledni souradnice cary
    # xv,yv = spoctene souradnice 
    upvar ${&xv} xv  ${&yv} yv
    set x0 [expr ([lindex $oval 0]+[lindex $oval 2]) /2 ] 
    set y0 [expr ([lindex $oval 1]+[lindex $oval 3]) /2 ]
    set a [expr $x0 - [lindex $oval 0]]
    set b [expr $y0 - [lindex $oval 1]]
    set x1 [expr $lx - $x0]
    set y1 [expr $ly - $y0]
    set k [expr $y1/$x1]
    set Ds [expr sqrt($a*$a*$k*$k + $b*$b)]
    set xv [expr abs($a*$b/$Ds)]
    if {$x1 < 0} { set xv [expr - $xv]}
    set yv [expr int($k*$xv + $y0)]
    set xv [expr int($xv+$x0)]
}

proc kdIntersecPoly {poly line &xv &yv} {

}

proc kdIntersecRec {rec line &xv &yv} {
    upvar ${&xv} xv
    upvar ${&yv} yv
    seteach {rx1 ry1 rx2 ry2} $rec
    seteach {lx1 ly1 lx2 ly2} $line
    set k [expr ($ly2-$ly1)/($lx2-$lx1)]
    foreach xx [list $rx1 $rx2] {
	if {($xx >= $lx2 && $xx <= $lx1) || \
		($xx <= $lx2 && $xx >= $lx1)} {
	    set yv [expr int($ly1+$k*($xx-$lx1))]
	    if {$yv >= $ry1 && $yv <= $ry2} {
		set xv [expr int($xx)]
		return
	    }
    }   }
    foreach yy [list $ry1 $ry2] {
	if {($yy >= $ly2 && $yy <= $ly1) || \
		($yy <= $ly2 && $yy >= $ly1)} {
	    set xv [expr int($lx1+($yy-$ly1)/$k)]
	    if {$xv >= $rx1 && $xv <= $rx2} {
		set yv [expr int($yy)]
		return
	    }
    }   }
}

proc seteach {vars vals} {
    set i 0
    foreach p $vars {
	upvar $p P
	set P [lindex $vals $i] 
	incr i
    }
}

proc swap_coords {coords} {
    return [concat [lrange $coords 2 3] [lrange $coords 0 1]]
}

proc regFilter {filter list} {
    set ret ""
    foreach l $list {
	if [regexp $filter $l] {
	    lappend ret $l
    }   }
    return $ret
}
    

proc kdCreateBox {w x y tag {col red}} {
    $w create rec [expr $x -2] [expr $y -2] [expr $x +2] [expr $y +2] \
	    -outline $col -fill $col -tag $tag
}


proc readFile file {
    set fid [open $file r]
    set ret [read $fid]
    close $fid
    return $ret
}

proc autoIncrementProc {var idx mode} {
    global $var
    if [array exists $var] {
	append var ($idx)
    }
    incr $var
}

proc autoIncrement var {
    regsub {\(.*\)} $var "" varname
    global $varname
    set $var -1
    trace variable $var r autoIncrementProc
}

